function [Simu] = Gibbs_regression(y,X,nb_MCMC,prior_choice,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tutorial 2 : Direct simulation and Gibbs samplers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the dependent variable (T * 1)
%%%%% X stands for the explanatory variables (T * k)
%%%%% nb_MCMC : number of MCMC iterations 
%%%%% prior_choice : if =1, Gibbs sampler under NIP; 
%%%%% if =2, prior 2: IG2 for sigma2, CP for b1, NIP for other b,; 
%%%%% if =3,Normal prior for b1, NIP for other b and for sigma2;
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% DS_beta and DS_sigma : Direct sampling from the posterior
%%%%% distribution under NIP priors.
%%%%% post_beta and post_sigma : MCMC Posterior draws of Beta and sigma^2.
%%%%% beta_ols and sigma_ols are the OLS results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<5)
    graph=0;
end
%%%% By default, set the prior_choice to the NIP.
if(prior_choice<1 || prior_choice>3 || nargin<4)
    prior_choice = 1; 
end
%%%% Check if the variables y and X exhibit an appropriate size.
T = max(size(y));
dimension = min(size(X));
if(dimension~=size(X,2))
    X = X';
end
%%%% Compute the OLS estimator
beta_ols = (X'*X)\X'*y;
SSR = (y-X*beta_ols)'*(y-X*beta_ols);
sigma_ols = SSR/(T-dimension);
%%%%%%%%%%%%%%%%%%
%%%% Direct Sampling of posterior when NIP
%%%%%%%%%%%%%%%%%%
% disp('**********************************');
% disp('******** DIRECT SAMPLING  ********');
% disp('**********************************');
% inv_X_X = inv(X'*X);
% Simu.DS_sigma = zeros(nb_MCMC,1);
% Simu.DS_beta = zeros(dimension,nb_MCMC);
% for i=1:nb_MCMC
%    sigma = 1/gamrnd((T-dimension)/2,2/SSR);
%    Simu.DS_beta(:,i) = mvnrnd(beta_ols,sigma*inv_X_X)';
%    Simu.DS_sigma(i) = sigma;
% end

%%%%%%%%%%%
%%%% MCMC Initialisation
%%%%%%%%%%%
burn_in = ceil(0.3*nb_MCMC);
nb_post = nb_MCMC-burn_in;
beta_post = zeros(dimension,nb_post);
sigma_post = zeros(nb_post,1);

%%%% Set the prior parameters
if(prior_choice==1)
    %%%%%%
    %%% Prior 1 : Non-informative prior
    %%%%%
    s_0 = 0;
    v_0 = 0;
    beta_0 = zeros(dimension,1);
    M_0 = zeros(dimension);
elseif(prior_choice==2)
    %%%%%%
    %%% Prior 2 : Informative conjugate prior for b1 and sigma2, NIP for
    %%% other coefficients
    %%%%%%
    s_0 = (y-X*beta_ols)'*(y-X*beta_ols)/2; %% IG_2(a,b) = IG(a/2,b/2)
    v_0 = 3/2;
    beta_0 = zeros(dimension,1);
    M_0 = zeros(dimension);
    M_0(2,2) = (0.1^2)^(-1)*s_0/(v_0-1); %% slide 112 but with a IG instead of a IG_2 ( var(beta) = E(var(beta|sigma^2)) + Var(E(beta|sigma^2)) = M_0^{-1}*s_0/(v_0-1) )
else
    %%%%%%
    %%% Non-informative prior (NIP) except b_1 \sim N(0,0.1^2)
    %%% NIP also for sigma2
    %%%%%
    s_0 = 0;
    v_0 = 0;
    beta_0 = zeros(dimension,1);
    M_0 = zeros(dimension);
    M_0(2,2) = 1/(0.1^2);
end

%%%%%%%%%%%%%%%%%%%%%
%%%% Conditional expectation (mu_cond) of beta|Sigma is independent of Sigma and can
%%%% be computed before the MCMC and not at each iteration
%%%%%%%%%%%%%%%%%%%%%
if(prior_choice~=3)
    Sigma_cond = M_0 + X'*X;
    mu_cond = (X'*y + M_0*beta_0);
    Sigma_cond = inv(Sigma_cond);
    mu_cond = (M_0 + X'*X)\(mu_cond);
end

disp('**********************************');
disp('***** BEGINNING OF THE MCMC ******');
disp('**********************************');
%%%%%%%%%%%%%%%
%%%% MCMC starting values
%%%%%%%%%%%%%%%
beta = beta_ols;
sigma = sigma_ols;
iter = 1;
for i=1:nb_MCMC
    progressbar(i/nb_MCMC);
    %%%%%%%%%%%
    %%%% Saving posterior samples
    %%%%%%%%%%%
    if(i>burn_in)
        beta_post(:,iter) = beta;
        sigma_post(iter) = sigma;
        iter = iter +1;
    end
    
    %%%%%%%%%%%%%
    %%% Update of sigma|beta
    %%%%%%%%%%%%%
    if(prior_choice~=2)
        d = 0.5*((y-X*beta)'*(y-X*beta) + (beta-beta_0)'*M_0*(beta-beta_0)) + s_0;
        c = 0.5*(T) + v_0 + (prior_choice==1)*(1/2);
    else
        d = 0.5*((y-X*beta)'*(y-X*beta));
        c = 0.5*(T);
    end
    inv_sigma = gamrnd(c,1/d);
    sigma = 1/inv_sigma;
    
    
    %%%%%%%%%%%%%
    %%% Update of beta|sigma
    %%%%%%%%%%%%%
    if(prior_choice~=3)
        beta = mvnrnd(mu_cond,Sigma_cond*sigma)';
    else
        Sigma_cond = M_0*sigma + X'*X;
        mu_cond = (X'*y + M_0*beta_0);
        Sigma_cond = inv(Sigma_cond);
        mu_cond = (sigma*M_0 + X'*X)\(mu_cond);
        beta = mvnrnd(mu_cond,Sigma_cond*sigma)';
    end
end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%
Simu.post_beta = beta_post;
Simu.post_sigma = sigma_post;;
Simu.beta_ols = beta_ols;
Simu.sigma_ols = sigma_ols;
Simu.post_mean_beta = mean(beta_post,2);
Simu.post_std_beta = std(beta_post,0,2);
post_var_beta_rao = zeros(size(Sigma_cond,1));
for i=1:nb_post
    if(prior_choice==3)
        Sigma_cond = M_0*sigma_post(i) + X'*X;
        Sigma_cond = inv(Sigma_cond);
    end
    post_var_beta_rao = post_var_beta_rao + Sigma_cond*sigma_post(i);
end
Simu.post_var_beta_rao_blackwellisation = post_var_beta_rao/nb_post;
Simu.post_var_beta = cov(beta_post');
Simu.post_mean_sigma = mean(sigma_post,1);
Simu.post_std_sigma = std(sigma_post,0,1);

%%%%%%%%%%%%%
%%% Testing for the convergence by Geweke's test, Cusum plot and ACF
%%%%%%%%%%%%%
Simu.test_Geweke = Geweke_test(beta_post);
[Simu.cusum] = cusum_plot(beta_post,graph);
if(graph==1)
    figure;
    for z=1:dimension
        subplot(ceil(0.5*dimension),2,z),autocorr(beta_post(z,:),200),title(['Sample ACF of \beta : ' num2str(z)]);
    end
    figure;
    autocorr(sigma_post,200),title('Sample ACF of \sigma^2');
end